function dag_to_dot(G, varargin)
% DAG_TO_DOT Make a file representing the directed graph in dotty format.
% dag_to_dot(G, ...)
%
% Optional arguments should be passed as name/value pairs [default]
%
% 'filename' - if omitted, we write to 'tmp.dot', convert this to 'tmp.ps',
%              and then call ghostview automatically 
% 'arc_label' - arc_label{i,j} is a string attached to the i->j arc. [""]
% 'node_label' - node_label{i} is a string attached to node i. ["i"]
% 'width'      - width in inches [10]
% 'height'     - height in inches [10]
%
% For details on dotty, See http://www.research.att.com/sw/tools/graphviz

% set default args
filename = [];
node_label = [];
arc_label = [];
width = 10;
height = 10;

% get optional args
args = varargin;
for i=1:2:length(args)
  switch args{i}
   case 'filename', filename = args{i+1};
   case 'node_label', node_label = args{i+1};
   case 'arc_label', arc_label = args{i+1};
   case 'width', width = args{i+1};
   case 'height', height = args{i+1};
  end
end


if isempty(filename)
  make_file(G, 'tmp.dot', node_label, arc_label, width, height);
  !dot -Tps tmp.dot -o tmp.ps
  !ghostview tmp.ps &
else
  make_file(G, filename, node_label, arc_label, width, height);
end


%%%%%%

function make_file(G, filename, node_label, arc_label, width, height)

n = length(G);
fid = fopen(filename, 'w');
fprintf(fid, 'digraph G {\n');
fprintf(fid, 'center = 1;\n');
fprintf(fid, 'size=\"%d,%d\";\n', width, height);
for i=1:n
  if isempty(node_label)
    fprintf(fid, 'n%d;\n', i);
  else
    fprintf(fid, 'n%d [ label = "%s" ];\n', i, nodelabel{i});
  end
end
for i=1:n
  cs = children(G,i);
  for j=1:length(cs)
    c = cs(j);
    if isempty(arc_label)
      fprintf(fid, 'n%d -> n%d;\n', i, c);
    else
      fprintf(fid, 'n%d -> n%d [label="%s"];\n', i, c, arclabel{i,c});
    end
  end
end
fprintf(fid, '\n}');
fclose(fid);
