function h=gaussplot(mu, Sigma, ncontours, scale, linespec, ngrid)
% GAUSSDRAW Plot some contours of constant probability density of a 2D Gaussian centered on the mean
% h=gaussdraw(mu, Sigma, ncontours, scale_factor, linespec)
%
% Optional arguments [default]
% If ncontours = -1, we use the number provided by contour [1]
% We evaluate the Gaussian mu +- scale_factor*Sigma [sf=1]
% linespec is an optional line style argument ['k']
% ngrid = how many points to evaluate the Gaussian on (more = smoother curves but slower) [50]

if nargin < 3, ncontours = -1; end
if nargin < 4, scale = 1; end
if nargin < 5, linespec = 'k'; end
if nargin < 6, ngrid = 60; end

% Modelled afer demgauss from netlab.
sx = scale*Sigma(1,1);
sy = scale*Sigma(2,2);
Xmin = mu(1)-sx; Xmax = mu(1)+sx; 
Ymin = mu(2)-sy; Ymax = mu(2)+sy; 
Xvals = linspace(Xmin, Xmax, ngrid);
Yvals = linspace(Ymin, Ymax, ngrid);
[X1, X2] = meshgrid(Xvals, Yvals);
XX = [X1(:), X2(:)];
% the i'th row of XX is the (x,y) coord of the i'th point in the raster scan of the grid

probs = gauss(mu, Sigma, XX);
probs = reshape(probs, ngrid, ngrid);

if ncontours==-1
  [C,h]=contour(X1, X2, probs);
else
  [C,h]=contour(X1, X2, probs, ncontours, linespec);
end
 
