function [engine, loglik] = enter_evidence(engine, evidence, varargin)
% ENTER_EVIDENCE Add the specified evidence to the network (jtree_dbn)
% [engine, loglik] = enter_evidence(engine, evidence, ...)
%
% evidence{i,t} = [] if if X(i,t) is hidden, and otherwise contains its observed value (scalar or column vector)
%
% The following optional arguments can be specified in the form of name/value pairs:
% [default value in brackets]
%
% maximize - if 1, does max-product instead of sum-product [0]
%
% e.g., engine = enter_evidence(engine, ev, 'maximize', 1)


% for add_ev in marginal_nodes
T = size(evidence, 2);
engine.evidence = evidence;
bnet = bnet_from_engine(engine);
engine.node_sizes = repmat(bnet.node_sizes_slice(:), [1 T]);

maximize = 0;
filter = 0;

% parse optional params
args = varargin;
nargs = length(args);
if nargs > 0
  for i=1:2:nargs
    switch args{i},
     case 'maximize', maximize = args{i+1}; 
     case 'filter',  filter = args{i+1};
     otherwise,  
      error(['invalid argument name ' args{i}]);       
    end
  end
end

assert(~filter);
engine.maximize = maximize;
engine.jtree_engine = set_fields(engine.jtree_engine, 'maximize', maximize);
engine.jtree_engine1 = set_fields(engine.jtree_engine1, 'maximize', maximize);

[ss T] = size(evidence);
engine.T = T;
observed_bitv = ~isemptycell(evidence);
onodes = find(observed_bitv);
pot_type = determine_pot_type(bnet, onodes); % we assume we can use the same pot_type in all slices
CPDpot = convert_dbn_CPDs_to_pots(bnet, evidence, pot_type);
[engine.clpot, loglik] = enter_soft_evidence(engine, CPDpot, observed_bitv, pot_type);
