function marginal = marginal_nodes(engine, query, add_ev)
% MARGINAL_NODES Compute the marginal on the specified query nodes (stab_cond_gauss)
% marginal = marginal_nodes(engine, query, add_ev)
%
% 'query' must be a singleton set.
% add_ev is an optional argument; if 1, we will "inflate" the marginal of observed nodes
% to their original size, adding 0s to the positions which contradict the evidence

if nargin < 3, add_ev = 0; end

bnet = bnet_from_engine(engine);
nclq = length(engine.cliques);

b_marg = 0;
for i = 1:nclq
  if mysubset(query, engine.cliques{i})
    pot = struct(engine.clpot{i});
    %if mysubset(query, pot.cheaddom) | mysubset(query, pot.ddom)
    if mysubset(query, pot.domain)
      marginal = marginal_singleclq_nodes(engine, i, query);
      b_marg = 1;
      break;
    end
  end
end

if ~b_marg 
    if length(query) == 1
        c = clq_containing_nodes(engine, query);
        if c == -1
            error(['no clique contains ' num2str(query)]);
        end
        marginal.domain = query;
        marginal.mu = [];
        marginal.Sigma = [];
        marginal.T = 1;
    else
        marginal = marginal_difclq_nodes(engine, query);
    end
end

if add_ev
  bnet = bnet_from_engine(engine);
  %marginal = add_ev_to_dmarginal(marginal, engine.evidence, bnet.node_sizes);
  marginal = add_evidence_to_gmarginal(marginal, engine.evidence, bnet.node_sizes, bnet.cnodes);
end
