function engine = var_elim_inf_engine(bnet, varargin)
% VAR_ELIM_INF_ENGINE Variable elimination inference engine
% engine = var_elim_inf_engine(bnet, ...)
%
% The variable elimination algorithm (also known as bucket elimination, or peeling)
% "pushes sums inside of products" as far as possible, and then works backwards,
% performing the summations, and putting the intermediate terms in the scope of the correct sum (bucket).
% The complexity depends on the summation order. In this implementation, we make no attempt to
% optimize this (which is an NP-hard problem).
% By using potential objects, instead of working directly with multidimensional arrays, this implementation
% also handles Gaussian networks.
%
% For details, see
% - R. Dechter, "Bucket Elimination: A Unifying Framework for Probabilistic Inference", UA1 96, pp. 211-219. 
% - Z. Li and B. D'Ambrosio, "Efficient inference in Bayes networks as a combinatorial
%     optimization problem", Intl. J. Approximate Reasoning, 11(1):55-81, 1994
% - R. McEliece and S. M. Aji, "The Generalized Distributive Law", IEEE Trans. Inform. Theory, 46(2), 2000


if 0
maximize = 0;
if nargin >= 2
  args = varargin;
  nargs = length(args);
  for i=1:2:nargs
    switch args{i},
     case 'maximize', maximize = args{i+1}; 
     otherwise,  
      error(['invalid argument name ' args{i}]);       
    end
  end
end
end

% This is where we will store the results between enter_evidence and marginal_nodes
engine.evidence = [];
engine.maximize = [];

engine = class(engine, 'var_elim_inf_engine', inf_engine(bnet));
