
% inner function which updates various boosting parameters, used by DistBoost.m

function [newW,newUnlabeledW,hypWeight,finalHyp,hypCorr,SupervisedWeight,SupervisedError, stop_flag ]= ...
    boostConstraintsOpt(oldW,oldUnlabeledW,marginMat,PosInds,NegInds,finalHyp,BSParams)

stop_flag=0;
unsignedMarginMat= marginMat;

%compute labeled pairs margin
labelsMargin=[ marginMat(PosInds) ; -marginMat(NegInds) ];
labelsWeights=[ oldW(PosInds) ; oldW(NegInds) ];

% compute unlabeled pairs margin
marginMat=abs(marginMat);   % unconstrained points

hypCorr= (labelsWeights'*labelsMargin)/sum(labelsWeights);  %r in singer notations

%check to see if hypothsis is bad!
if hypCorr<0
    stop_flag=1;
    disp(sprintf('bad correlation %d',hypCorr));
end  

% step size (alpha) determination
if BSParams.simple_alpha==1 %this is the standard AdaBoost update
    hypWeight= 0.5*log( (1+hypCorr)/(1-hypCorr) ); % alpha in singer notation

else %optimize alpha using line search:
    if any( labelsMargin <0)& any(labelsMargin>0 )
        global Weights;
        global Margs;
        Weights=labelsWeights;
	Margs=labelsMargin;
	[hypWeight,fval,exitflag,output] = ...
	    fminbnd('calcZ',-10,10,optimset('TolX',0.01,'Display','off'));

	if abs(hypWeight)==10
	  disp('problem in of boostConstraints');
	end
    else
      disp('no negative margins at all line 53 of boostConstraints');
        hypWeight= 0.5*log( (1+hypCorr)/(1-hypCorr) ); % simple alpha
    end
end  

%now update FinalHyp:
finalHyp= finalHyp + hypWeight.*unsignedMarginMat;


% update unlabeled weights :
newUnlabeledW= oldUnlabeledW.*exp(-hypWeight*BSParams.Lambda);


% update labeled weights :
newW= oldW; %newly added.
newW([PosInds ; NegInds])=labelsWeights.*exp(-hypWeight*labelsMargin);

%normalize:
normalizer= (sum(sum(newW))+newUnlabeledW*(sum(sum(~newW))));
newW= newW./normalizer;
newUnlabeledW= newUnlabeledW./normalizer;

%supervisedWeight is the total weight of supervised constraints
SupervisedWeight=sum(newW([PosInds' NegInds']));

% SupervisedError is the weighted error over constraints
tmp=1 - (sign(labelsMargin)+1)/2;
SupervisedError=tmp'*newW([PosInds ; NegInds])/SupervisedWeight;

