
function [ TRMAP, TSMAP , MarginMap] = ...
    calcMarginMap2(TrainData,TestData,param,TypeFlag)

% CALCMARGINMAP2 inner function used by computeDistances.m computes the margin map between two
% sets of data points, and not a distances between a set and 
% itself. It is required in the context of retrieval evaluation. 
% 
% Description:
%         [ TRMAP, TSMAP , MarginMap] = calcMarginMap2(TrainData,TestData,param,TypeFlag)
%
% parameters:
% 
%         TrainData =   The data 'in the data base'
%         TestData  =   The data 'presented as queries'
%         param     =   The current GMM model used (or weak learner).
%
%         TypeFlag  =   Type of margin : 0 (default) - Map margin
%                             1 - probabilistic margin
%                             2 - margin as difference between MAP and the second best.
%
%returns:
%
%        TRMAP      =   MAP assinment ( unconstrained ) of train data points.
%        TSMAP      =   The same for TestData.
%        marginMap: =   Matrix of length(TrainData)*length(TestData) which contains 
%                       the pairwise confidence of the MAP assignment. if pos then points belong
%                       together, if neg, points do not belong together in MAP assignment.
%
%----------------------------------------------------------------------------------------------------

[M , stam ]= size(param);
singleCovMatFlag=1;
if isempty(param(2,2))
    singleCovMatFlag=0;
end
if ~exist('TypeFlag','var')
    TypeFlag=0;
end

[nTrain d]= size(TrainData);
nTest=size(TestData,1);

data=[ TrainData ; TestData ];
n=nTrain+nTest;

for j=1:M
    if (singleCovMatFlag == 1)
        cov_index=1;
    else
        cov_index=j;
    end
    tempData=data-ones(n,1)*param{j,1}; % subtract the mean
    q1=tempData*inv(param{cov_index,2}).*tempData;
    expContent=-0.5*sum(q1,2);         % the exponent content
    LogProbs(:,j)= -(d/2)*log(2*pi) -0.5*log(det(param{cov_index,2}))+expContent;
    LogProbs(:,j)=LogProbs(:,j)+log(param{j,3});
end

m=max(LogProbs');
LogProbs=LogProbs- m'*ones(1,M);
probabilities=exp(LogProbs);
%normalize probabilities:
probabilities= probabilities./(sum(probabilities')'*ones(1,M));

TrainProbs=probabilities(1:nTrain,:);
TestProbs=probabilities(nTrain+1:nTrain+nTest,:);

[ TRProbs , TRMAP ] = max(TrainProbs');
[ TSProbs , TSMAP ] = max(TestProbs');
if TypeFlag==0
    MarginMap=TRProbs'*TSProbs;
    SignMap=( ( TRMAP'*ones(1,nTest) == ones(nTrain,1)*TSMAP )*2 )-1;
   MarginMap=MarginMap.*SignMap;
elseif TypeFlag==1
    MarginMap=TrainProbs*TestProbs';
    MarginMap=(MarginMap-0.5)*2;
elseif TypeFlag==2
    %mult1 = repmat(probabilities',M,1);
    plusInds=1:M+1:M^2;
    MinusInds=setdiff(1:M^2,plusInds);
    TRmult=repmat(TrainProbs',M,1);
    for i=1:nTest
        TSmult=reshape(repmat(TestProbs(i,:),M,1),M^2,1)*ones(1,nTrain);
        pairProbs=TRmult.*TSmult;
        BestPlus=max(pairProbs(plusInds,:));
        BestMinus=max(pairProbs(MinusInds,:));
        MarginMap(:,i)=( BestPlus-BestMinus )';
    end
else
    MarginMap=( ( TRMAP'*ones(1,nTest) == ones(nTrain,1)*TSMAP )*2 )-1;
end
