
function [outputMat] = computeDistances(Hyp,data1,data2,similarityFlag)

% COMPUTEDISTANCES - This function computes the distances between all pairs of points in data1
%                    and data2 using a 'voting of gaussian mixtures' schema as a distance function.
%
%     Description:
%                  [outputMat] = computeDistances(Hyp,TrainData,TestData,hypFlag,similarityFlag)
%
%     parameters:
%   
%                  Hyp - a struct which contains the differnet hyp parametres- MOGs, and weights.
%                 
%                  data1 =  The first data matrix data, each datapoint is a row vector.
% 
%                  data2  =  The test data matrix data, each datapoint is a row vector.
%
%                  similarityFlag = If set to 1 returns similarity values, if set to 0 (default) returns distance values.
%
%    returns: 
%
%                  outputMat =  A distance or simialrity matrix of train vs. test data (depending on similarityFlag)
%                               which contains the initial hypothesis, or final hypothesis distance matrix
%                               (depending on hypFlag) 
%
%--------------------------------------------------------------------------------------------------------------------


if(~exist('similarityFlag','var'))
  similarityFlag=0;
end

[nTrain d]= size(TrainData);
nTest=size(TestData,1);
computedHyp= zeros(nTrain,nTest);



for t=1:length(Hyp.hypWeight)
  
  [ probabilities, mapAssignment, marginMap] = ...
      calcMarginMap2(TrainData,TestData,Hyp.MogParams{t},Hyp.MarginType);
  
  computedHyp= computedHyp + Hyp.hypWeight(t).*marginMap;
  
end

%check whether to output similarity values or distance values (both bounded between 0 to 1).
if(similarityFlag)
  [outputMat]= getBoundedSimilarity(computedHyp);

else %distance matrices
  [outputMat]= getBoundedDistance(computedHyp);
end

