
function [BSParams]= initBSParams(BSParams)

% INITBSPARAMS - sets various boosting parameters to their default values, only if they have not been manually set. 
%       fields set are:
% 
%   Description:
%          [BSParams]= initBSParams(BSParams)
%
%   parameters:
%
%          WeakParams =      The weak learner parmeters - see weakLearner for details. some parameters
%                            are set here for details see initWeakParams.m. note that some weak
%                            params are set here to create a darefull weak learner.
%
%          T =               The number of boosting iterations (default value set to 15).
%
%          numberOfCalls =   The number of attempts to recall the weak learner in case it
%                            fails to produce a valid hypothesis. (default value set to 8)
%
%          simple_alpha =    what is this!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1 default value set to 1
%
%          Lambda =          default value set to 1
%
%          record_flag =     Which boosting hypothesis should be recorded (optional). Default value set to 2.
%
%    
%          evaluateHypFlag =  If set to 1 will compute some evaluation statistics (Cumulative
%                            neighbor purity and KNN error for various K values). Default is set to 0.
%
%          evolutionFlag =   If set to 1 will collect all distance matrices of the boosting
%                            iterations to allow seeing the distance function evolve over time.
%
%          printStatsFlag =  If set to 1 will print some boosing development stats during
%                            training and when training is done. Default set to 1.
%
%          similarityFlag =  If set to 1 will output similarity matrices bounded between 0 to 1.
%                            otherwise outputs a bounded distance matrix between 0 to 1. 
%                            Default is 0 (i.e by default returns a distance matrix).
%   returns:
%
%          BSParams       =  The input struct with all fields initialized to input values, or
%                            default values.
%------------------------------------------------------------------------------------------------------------------

if(~isfield(BSParams,'T'))
  BSParams.T=15;
end

if(~isfield(BSParams,'numberOfCalls'))
  BSParams.numberOfCalls= 8;
end

if(~isfield(BSParams,'simple_alpha'))
  BSParams.simple_alpha=1;
end

if(~isfield(BSParams,'Lambda'))
  BSParams.Lambda=1;
end

if(~isfield(BSParams,'evaluateHypFlag'))
  BSParams.evaluateHypFlag=0;
end

if( (BSParams.evaluateHypFlag==1) & (~isfield(BSParams,'Knn')) )
  BSParams.Knn=[1];
end  

if(~isfield(BSParams,'evolutionFlag'))
  BSParams.evolutionFlag=0;
end

if(~isfield(BSParams,'printStatsFlag'))
  BSParams.printStatsFlag=1;
end

if(~isfield(BSParams,'similarityFlag'))
  BSParams.similarityFlag=0;
end




%some more parameters which should in general not be modified!
BSParams.MarginType=0;  %TypeFlag : Type of margin  0 - map margin 
%                           1 - correlation margin
%                           2 - first vs second margin
%                           3 - 0-1 MAP based margin 

BSParams.UMarginType=0;  %UnsupervisedMarginType :
%                                     for boosting 1 :
%                                     0 - local certainty encourage (abs)
%                                     1 - global certainty encourage (sign(cummulative hypothesis)*signed margin of weak hypothesys ) 
%                                     2 - unsupervised margin is lambda bigger ( unsupervised
%                                     pairs are decaying faster )
%                                     


