
function [WeakParams]= initWeakParams(WeakParams,labels)

% INITWEAKPARAMS sets various weak learner parameters to values suited for running DistBoost. 
% 
%   Description: 
%       [WeakParams]= initWeakParams(WeakParams,labels)
%       
%       Most of these valuse should not be modified, except for the following:
%
%          num_itertations = The number of EM rounds that the weak learner runs. This should be
%                            kept low in order to prevnet overfitting the weak learner, and to help it find diverse
%                            hyp. in each boosting round. default value is set to 6.
%         
%          M =               The number of models in the GMM. This should be >= to the true
%                            number of classes in the data. default is set to the true number of
%                            classes.
%
%   Note: many many other parameters are set to their default values in EMsetParams.m which is
%         used by the weak learner itself.
%------------------------------------------------------------------------------------------------------------------

if(~isfield(WeakParams,'M'))

  %determine number of models from labels if exist!
  if(exist('labels','var'))
    plg= pilug(labels);
    WeakParams.M= size(plg,1);
    
  else
    warning(['Number of models in the GMM (M) must be initialized in order to run properly, since' ...
	     ' labels have not been inputed, the number of models must be set directly throuhg' ...
	     ' BSParams.WeakParams.M, for now using default (meaningless) M=5.'])
  end
end

if(~isfield(WeakParams,'num_itertations'))
  WeakParams.num_iterations=6 ;    %will only have num_iterations in each EM call.
end


%make sure that the weak learner chooses dareful hyps.
if(~isfield(WeakParams,'number_of_starts'))
  WeakParams.number_of_starts=3;     
end



%some parameters which should not be modified.
WeakParams.single_cov_mat_flag= 1; % will only use single cov mat for all Gaussians.
WeakParams.late_oracle= 1; 
WeakParams.param=[];
WeakParams.ClassLabels= labels;
WeakParams.ConstrainedMapFlag= 0; %do not use constrained MAP assignment
WeakParams.noiseParam=[50 50 50];


