
function [pointW]= translateWeights(pairW,unlabeledW,maxFlag)

% TRANSLATEWEIGHTS - Translates weights on pairs of data points to weights on single data points.
%
%   Description:
%
%      [pointW]= translateWeights(pairW,unlabeledW,maxFlag)
%
%   Parameters:
%
%      pairW       = Sparse matrix of pairwise weights only on the labeled pairs as outputed by the boosting algorithm.
%      unlabeledW  = Weight of the unlabeled points - all unlabeled points share the same weight.
%      methodFlag  = if set to 1 will use  maximal  weight of all pairs of the given data point as its weight.
%                    if set to 0 will use average weight of all pairs of the given data point. (default value).
%   Returns:
%      pointW      = weights on single data points.
%
%----------------------------------------------------------------------------------------------------------------------

if(~exist('maxFlag','var'))
  maxFlag=0;
end

[N]= size(pairW,1);

%do we average over all data points, or just over constrained data points?
% currently average over all data points.
if(maxFlag)
  pointW= max(pairW); %unlabeled weights are always smaller than labeled weights.
else
  %also add weights of unlabeled points
  pointW= (sum(pairW) + sum(~pairW).*unlabeledW)./(N-1); %no need to average since we are normalizing the weights.
end

%normalize point weights to sum to N:
pointW= pointW./sum(pointW);
pointW= pointW.*N;
pointW= full(pointW); %just in case we get a sparse vector.
