function [stats]= collectStats(stats,t,finalHyp,PosInds,NegInds,hypWeight,hypCorr,EMZs,ConstraintsNum,ConstraintsNumNoDiag)

% COLLECTSTATS
%
%   Description: collect various boosting statistics.
% 
%   stats is a cell array in which:
%
%     stats{1} =  The Hypothsis weights     
%     stats{2} =  The weak hyp correlation (r).
%     stats{3} =  The weak learner Z scores. (combined precision recall score).
%     stats{4} =  The supervised weight fraction (out of the total sum of weights).
%     stats{5} =  The weighted error of weak Hypothses over supervised pairs.
%     stats{6} =  The average margin over supervised  pairs.
%     stats{7} =  The Commulative hypothesis error over supervised pairs.
%     stats{8} =  Average commulative margin of supervised constraints.
%     stats{9} =  Average commulative margin of unsupervised constraints.
%     stats{10} =  The Commulative hypothesis error over supervised pairs, not including diagonals.
%     stats{11} = The Commulative hypothesis Z score.
%     stats{13} = The average normilized margin (on both sup and unsup).
%     stats{14} = The total loss : sum of exps of margins.
%     stats{15} = The supervised loss : sum of exps of margins of supervised pairs.
%-----------------------------------------------------------------------------------------------------

[ stats{7}(t) , AlreadtCalculated , stats{9}(t), stats{13}(t) , stats{14}(t) ,stats{15}(t) ] =...
    evalGlobalMargin( finalHyp , PosInds, NegInds );


stats{1}=hypWeight;
stats{2}=hypCorr;
stats{3}=EMZs(3,:);

stats{13}(t)=stats{13}(t)/sum(hypWeight(1:t));
stats{10}(t)=stats{7}(t)*ConstraintsNum/ConstraintsNumNoDiag;

