% this function evaluates the quality of a mahalanovis distance over a data in terms of nearest neighbours criteria.

% inputs -  data
%           ClassSizes - the class sizes vector. classes are assumed to be present in the data in continuity and in the order
%           of ClassSizes
%           A - the Mahalanovis matrix to be assessed.

% returns:

%NeighborPurity - cell array of vectors of max number of neighbors
%which in the k'th index holds the purity of neighbors till the k'th
%neighobor.

% distances - the matrix of distances between data points 

function [ NeighborPurity , KNN , distances ]= evalMahalanobis(data,ClassSizes,A,Ks)

if ~exist('Ks','var')
    Ks=[];
end

[ n , dim ] = size(data);

% prepare distance matrix
distances=zeros(n,n);
for i=1:n
     
      temp=ones(n,1)*data(i,:);
      x_minus_y=temp-data;
      distances(i,:)= sum( ( ( x_minus_y*A ).*x_minus_y )' );
end

[ NeighborPurity , KNN]=evalDistances(distances , ClassSizes, Ks );
  
