% function [ chunks , hidden , ner ]=oracle_sim(qn,qs)
% this function simulates a random similarity oracle.
% It chooses at random qn groups of qs points each.
% this function is a random oracle of equivalence relations, using the
% 'distributed learning model'
% input :
% qn - number of queries
% qs - query size
% ClassLabels - a vector of the class labels.

% outputs :
% chunks - chunklets vector, Noam's format.
% map - a 'observable to hidden map', used by mixture 1-5
% o-ner -  a table on negative_number*2 of negative pairs. only a single
% negative is present between two chunklets.
% o-ner_all -  same as o_ner, but includes all the possible negatives. used
% by mixture6.

function [ chunks , map , o_ner , o_ner_all ]=randomOracleSim(qn,qs,ClassLabels)

n=length(ClassLabels);


% a map from data points to chunklets is being held and updated.
% map(i) is the chunkl of point i
map=1:n;		% map is initialized. 
lc=n+1;		% the chunklet label counter is starting at n+1, since at initialization 
% 1..n are occupied by the identity map.
ner=[ -3 -3 ];		% a stam initiaqlization, just to not be empty  
perm=randperm(n);
ic=1;
for i=1:qn
    
    % random qs points
    ic=ic+1;
    if ( (ic*qs) <= length(perm) )
      rinds=perm( (ic-1)*qs+1:ic*qs );
    else
      ic=1;
      perm=randperm(n);
    end
    labels=ClassLabels(rinds);
    tb=pilug(labels);
    
    % collect positive constraints and update map
    
    for j=1:size(tb,1)
        if tb(j,2)>1	% a local chunklet was found
            cinds=rinds( find(labels==tb(j,1)) );	% the chunklet's points indexes
            noup=length( find( unique(map(cinds))>n ) );	% number of unencountered points.
            if noup==0		% a new chunklet is created
                map(cinds)=lc;
                ner( ismember(ner,cinds) )=lc;	% update ner ( pos-neg closure )
                lc=lc+1;
            elseif noup==1	% an existing chunklet is extended
                rl=unique( map( cinds( find(map(cinds)>n) ) ) );
                ner( ismember( ner, map(cinds) ) )=rl;	
                map(cinds)=rl;
            else	% existing chunklets are unioned ( into the first one )
                uc=unique(map(cinds));
                rl=max(uc);	% the label for the joint chunklet ( 'max' isn't very important
                iic=find( ismember( map, uc ));
                ner( ismember(ner,map(iic)) )=rl;	
                map(iic)=rl;
            end
        end
    end
    
    
    % add negative constraints
    
    tb=pilug(map(rinds));
    s=size(tb,1);
    tmp=[ reshape(repmat(1:s,s,1),s^2,1) repmat((1:s)',s,1 )];
    tmp=tmp( setdiff(1:s^2,1:s+1:s^2),:);
    tmptb=tb(:,1);
    tmp=tmptb(tmp);
    tmp=tmp(find(tmp(:,1)<tmp(:,2)),:);
    ner=[ ner ; tmp ];
    
end

% unique Ner
ner=ner(2:end,:);		% get rid of the initialization
ner=unique(ner,'rows');

% create chunks from map
chunks=map;
chunks(chunks<=n)=-1;
tb=pilug(chunks);
for i=2:size(tb,1);
    chunks(chunks==tb(i,1))=i-1;
end

% transform ner into original points indexes.

% create reduced table of negatives
o_ner=[];
for i=1:size(ner,1);
    k=find(map==ner(i,1));
    j=find(map==ner(i,2));
    o_ner(i,:)=[ k(1),j(1) ];
end

% create full table of negatives
o_ner_all=[];
for i=1:size(ner,1);
    k=find(map==ner(i,1));
    j=find(map==ner(i,2));
    tmp1=reshape( repmat( k,length(j),1 ),length(k)*length(j),1);
    tmp2=repmat( j',length(k),1);
    o_ner_all=[ o_ner_all ; [ tmp1 , tmp2 ]];
end

















