%computes weighted RCA of data where NOT all data points belong to chunklets.
% uses a predetermined chunklet selection inputed by chunkletAssignments.


%params:
% TrainingSet - the data points, each data point is a row in the matrix.

% chunkletAssignments - the indices of points belonging to chunklets on
%                       current realization  - in the original data representation. 
%                       vector of number of points, -1 denotes not in chunklet,
%                       1:numberOfChunklets marks which points belong to which chunklets.
%
% W -                   weight vector which associates a weight for each of the data points in the dataset.
%
% priorFlag -           if set to 1 will smooth the cov matrix to handle ill conditioned matrices. 
%                       DEFAULT IS SET TO 1.

%returns
% A - the RCA suggested mahalanobis matrix.
% WhitenedData - the data after the RCA transformation.


function [A, WhitenedData] = weightedRCA(TrainingSet,chunkletAssignments,W,priorFlag)

if(~exist('priorFlag','var'))
  priorFlag=1;
end


TrainingSet=TrainingSet';	% tomer conversion
S= max(chunkletAssignments);
ChunkletDataSet= [];
ChunkletSizes= [];
wChunks= []; %weight vector of points only belonging to chunklets.
for i=1:S
  inds= find(chunkletAssignments == i);
  ChunkletSizes(i)= length(inds);
  ChunkletDataSet= [ChunkletDataSet TrainingSet(:,inds)];
  wChunks= [wChunks W(inds)];
end


%now normalize wChunks to sum to 1: WHAT ABOUT THIS HO HO HO HO
%wChunks= wChunks./sum(wChunks);
wChunks= wChunks./sum(wChunks);
wChunks= wChunks*size(ChunkletDataSet,2);

[CenteredChunklets]=centerWeightedChunklets(ChunkletDataSet,ChunkletSizes,wChunks);

[dim N]= size(CenteredChunklets);

%introduce weights:
weightedCenteredChunklets= CenteredChunklets.*((wChunks'*ones(1,dim))');

%now compute weighted cov mat of chunklets.
Cov= weightedCenteredChunklets*CenteredChunklets';

%check for ill ranked RCA covmat in case of not enough chunklets: if
%so smooth matrix:
alpha= 1/50;
if(priorFlag)&(rank(Cov) < dim)
    Cov= (Cov + alpha.*eye(dim))/(1+alpha);
end


%compute the svd of the normalized chunklets
[U Sig V]= svd(Cov);

%whiten the data set using RCA Transformation:

Y= U'*TrainingSet;
Z= (Sig^-0.5)*Y;
WhitenedData= U*Z;

% the mahalanovis matrix
A=inv(Cov);
