% this version of mixture_of_gaussian can handle weights.
% The weights are introduced using the weights as chunklet sizes.

% the function arguments :

% data                      : the data - every row is a data point.
% k                         : the number of componenets.
% chunks                    : a label list containing the chunklet information (length =length(data) )
%                                -1  -  the point doesnt belong to any chunklet.
%                                 otherwise points with the same label are chunkled.
% anti _ chunks		    : a table(n,2) of pairs. each pair is anti chankletized.
% EMParams              : a structure containing parameters for the EM
%                       : all the parameters have defaults
%                       : the argument is optional
%                       : the EMParameters is fully documented in 
%                       : EMSetParams.m
%returns:

% results                   : the best parameters achieved
% logLikelihood             : a log of the ll decrease.
% groups                    : the prefered centers for the points.
% z                         : when z_record_flag=0 returns a vector of 3 numbers: which are
%                             purity, accuarcy and z_score, else the log of the z_score.

% exit_flags			bit 1 : the last round decrease the ll
%				bit 2 : obsolete
%				bit 3 : one model or more was removed
%				bit 4 : catastroph : ll reached -infinity or all models have been deleted.
%				bit 5 : a covmat was close to singular during the convergence.
%				bit 6 : a model was frozen while EM

function [results,logLikelihood ,groups ,z , anti_chunks_pruned ,exit_flags ]=...
   mixture_of_gaussians5(data, k, chunks, anti_chunks, EMParams)

global late_oracle;

EMsetParams;    % set the algorithm parameters.

% globals and initialization

global pruned_flag;   
pruned_flag=0;
iteration=1;     % iteration.
global nets;	% the markov nets ( global used in calculate_partition_function2 and in calc_p_and_pll)
global OriginalNets;		% stores the full original nets ( before prunning ) in case 
								% we want to re prune.
original_k=k;	% save for case of model degradation
last_iteration_missed=0;	% a flag indicating whether the last iteration decreased ll
z=[];		% z record ( purity-acuracy scores )
global chunklet_sizes;	% for chunklet pf calculation
global anti_chunk_num;	% for anti chunklet pf aproximation calculation
anti_chunk_num=size(anti_chunks,1);
global anti_chunks_pruned;  % meaningfull only when reduce_arcs==2. the number of pruned negative constraints
anti_chunks_pruned=0;
global exit_flags;	% error messages : details on function remark
exit_flags=0;
global frozen_models;	% used in maximize. a vector containing the indexes of models
								% that were frozed in this session ( not neccesarily that they are frozen now )
frozen_models=[];
EMParams.remove_models_flag=1;		% if small_model_policy is 1 this variable is 1 for 
                           % the first round, 0 for the other rounds.
grace_turn_flag=0;			% used when small model policy is 2, to indicate the grace turn.
                           
% divide the data into connected components and prepare graphs for each.
global HiddensW; % used in calc_p_and_pll2 when resetting PairsW after edges prunning. 
global SinglesW;
global PairsW;

%********************************************************************************

param=EMParams.param;   % set the start parameters

while (1)

  % E -step :
   % calculate the probabilities p( hidden i belongs to center j).
      
   if mm_flag 	% hard (MM) version
     [ probabilities , logLikelihood , ass ] =calc_ass_and_maxll(data,param,single_cov_mat_flag,...
						  c_inds,nc_inds,singles,lab_mat,colors,w);
   else		% soft (EM) version
     [probabilities, logLikelihood ]=calc_p_and_pll4(data,param,...
         EMParams.single_cov_mat_flag,c_inds,nc_inds,singles, ...
         EMParams.lab_mat,EMParams.colors,EMParams.w);
   end
   
   % a calculation of the partition function 
   [ logPF GPF ]=calculate_partition_function3(cell2mat(param(:,3))',1);
   
   
   logLikelihood = logLikelihood - logPF;
  
   % keeping purity acuracy record
   if (~isempty(EMParams.ClassLabels)) & (z_record_flag )
       if ~mm_flag
           if (EMParams.ConstrainedMapFlag)
               [ stam , stam , groups ] =calc_ass_and_maxll(data,param,single_cov_mat_flag,c_inds,nc_inds,singles,lab_mat,colors,w);
               groups=groups(oth);	 % from hidden assignment to observable assignment.
           else
               [stam, groups ]=calc_map_labeling1(data,param,k,EMParams.single_cov_mat_flag);
           end
       end
       ass=ass(oth);	% from hidden assignment to observable assignment.
      [a2 , b2]=purity_accuracy2(EMParams.ClassLabels,ass,original_k);
      %visual_k_means(data,EMParams.ClassLabels,ass);

      z(iteration)=2*a2*b2/(a2+b2);
   end

   %change tomer 25/6 - when ll goes to -inf stop run and exit with
   %exit flag 8
   if(logLikelihood == -inf)
       z=[-1; -1 ;-1];
       results=[];
       logLikelihood=[];
       groups=[];
       anti_chunks_pruned=[];
       exit_flags=bitor(exit_flags,8);
       return
   end
    
   % record loglikelihood and check for stop
   ll(iteration)=logLikelihood;
   
   if iteration>1
       if(num_iterations > 0)
           if(iteration > num_iterations) 
               break;
           elseif(isempty(chunks)) & ( (ll(iteration)-ll(iteration-1))/abs(ll(iteration)))<stop_saf	% stop condition
               break;
           end
       elseif  ( (ll(iteration)-ll(iteration-1))/abs(ll(iteration)))<stop_saf	% stop condition
           if ~grace_turn_flag
               break;
           else
               'grace turn saved me'
           end
       end
   end
   
   iteration=iteration+1;
   prev_param=param;			% save it for case of last terrible round. 

   % calculate the new parameters.

   param=maximize2(data,k,param,probabilities,oth,EMParams);

   if small_model_policy==2 & size(param,1)~=k	& iteration>2 % a model has been removed - give grace turn
      grace_turn_flag=1;
      iteration;
  else
      grace_turn_flag=0;
   end
        
   k=size(param,1);	% update the number of models ( models might have been removed ) 
   
   if iteration==2	% after the first iteration, models can be freezed
      if small_model_policy==1
         EMParams.remove_models_flag=0;	% in itrations >=2 no models are removed.
      end
   end

   % change noam 23.6
   if isempty(param) % meaning: maximize has deleted all models
     z=[-1; -1; -1];
     results=[];
     logLikelihood=[];
     groups=[];
     anti_chunks_pruned=[];
     exit_flags=bitor(exit_flags,8);
     return
   end  
   %%%% end change noam 23.6
   
   iteration;

end


% handling terrible bug ( which is expected at this version )
if ll(end)<ll(end-1)		 % the last iteration decreased ll 
    disp( sprintf('The %d iteration decreased ll from %d to %d \n',iteration,ll(end-1),ll(end) ));
   exit_flags=bitor(exit_flags,1);
   ll=ll(1:end-1);
   if z_record_flag
      z=z(1:end-1);
   end
   param=prev_param;
   last_iteration_missed=1;
end
%sprintf('%d iterations',iteration)

% last prepare the outputs : 

logLikelihood=ll;
results=param;

if (~z_record_flag) | (last_iteration_missed)	% need to caculate the final assignment
    if (EMParams.ConstrainedMapFlag)
        [ stam , stam , groups ] =calc_ass_and_maxll(data,param,single_cov_mat_flag,c_inds,nc_inds,singles,lab_mat,colors,w);
        groups=groups(oth);	 % from hidden assignment to observable assignment.
    else
        [stam, groups ]=calc_map_labeling1(data,param,k,EMParams.single_cov_mat_flag);
    end
else
   groups=ass;
end

if (~isempty(EMParams.ClassLabels)) & (~z_record_flag)		% calculate groups.
   z=zeros(3,1); % z(1)=purity %z(2)=accuracy z(3)=score 
  [purity , accuracy]=purity_accuracy2(EMParams.ClassLabels,groups,original_k);
  z(1)=purity;
  z(2)=accuracy;
  z(3)=2*purity*accuracy/(purity+accuracy);
end

if(isempty(z))
  z=[-1 ; -1 ; -1];
end
