% this version of caculate_partition_function doesn't support exact
% computation of the PF using a markov network. ( since it is too
% complicated and slow ). instead , it supports better approximation in
% the case of combined, and supports weights of both kinds : pointwise and
% constraintswise. 

function [ logPF ,  GPF ] = ...
    calculate_partition_function3(a,no_grad_flag,aa_flag)
% a - the current cluster weights  ( 1*m matrix ) 
% no_grad_flag  - when set the gradient isn't caculated and the caculation time reduces by factor of 3*k
% impotant input parameters passed as globals are stage and late_oracle.
% S (singles) - list of the weights of single (unconnected ) hiddens ( 1*single num )
%           ( weights caused by chunklets or w .weights of 1 are not needed )
% P (pairs) - a matrix 2*anti_num with the weights of the two vertex of the anti.
%          pairs and singles are computed 

% logPF - the log of the partition function  
% gpf - the partition function gradient. the gradient is actually PF*Ggpf,
% but the direction is well kept by gpf alone, and PF is cancelled anyway
% in approximateWeights.

% first, the logPF is computed. I use here an extended approximation based on
% the 'disjoint in pairs' assumption.

global late_oracle;
global SinglesW;
global PairsW;
global aa_flag;
S=SinglesW;
P=PairsW;

if late_oracle==0   % early oracle
    logPF=0;
    GPF=0;
    return;
end

% should we compute anti partition function ?
if ~exist('aa_flag','var')
    aa_flag=0;
end

M=size(a,2);    % the number of models
Sn=size(S,2);   % number of singles
Pn=size(P,2);   % number of pairs

% make vector a contain a '1' component ( for numeric stability )
origA=a;
K=max(a);
a=a/K;

% compute logPF for single components
logPFComps_tag=log ( sum ( ( a'*ones(1,Sn) ).^( ones(M,1)*S ) ) );
logPFComps(1:Sn)=logPFComps_tag + log(K)*S;  % normalization compensation

% compute logPF for pairs components
if Pn~=0 & aa_flag
    side1= sum ( ( a'*ones(1,Pn) ).^( ones(M,1)*P(1,:) ) );
    side2= sum ( ( a'*ones(1,Pn) ).^( ones(M,1)*P(2,:) ) );
    mixed= sum ( ( a'*ones(1,Pn) ).^( ones(M,1)*sum(P) ) );
    logPFComps_tag(Sn+1:Sn+Pn) = log(side1.*side2-mixed);
    logPFComps(Sn+1:Sn+Pn)= ...
        logPFComps_tag(Sn+1:Sn+Pn)+sum(P)*log(K);   % normalization compensation
end

logPF=sum(logPFComps);   % logPF is a sum of logPFComps

if no_grad_flag==1
    GPF=[];
    return;
end

% computation of GPF - the gradient direction of PF.

% as a gradient of component product, GPF direction can be presented as weighted 
% sum of components gradients, while the weights are 1/component.
% to keep numerical stability, i therefore divide by the components by the
% smallest of them , makinf it's weight 1. this is done via subtraction of
% logarithmic expressions.

% compute components weights ( without the K factor ) 
GradCompWeights=exp(-logPFComps_tag);

% compute d/d_a for the PF components
Grads=zeros(M,Sn+Pn);

% single components :
Grads(:,1:Sn)= ( ( a'*ones(1,Sn) ).^( ones(M,1)*(S-1) ) ) .* ( ones(M,1)*S );

% pairs components :
if Pn~=0 & aa_flag
    vertex1= ( ( a'*ones(1,Pn) ).^( ones(M,1)*(P(1,:)-1) ) ) .* ( ones(M,1)*P(1,:) );
    vertex2= ( ( a'*ones(1,Pn) ).^( ones(M,1)*(P(2,:)-1) ) ) .* ( ones(M,1)*P(2,:) );
    vertex1=vertex1.*(ones(9,1)*side2);
    vertex2=vertex2.*(ones(9,1)*side1);
    mixed= ( ( a'*ones(1,Pn) ).^( ones(M,1)*(sum(P)-1) ) ) .* ( ones(M,1)*sum(P) );
    
    Grads(:,Sn+1:Sn+Pn)= vertex1+vertex2 -mixed;
end

GPF=sum(Grads.*( ones(M,1)*GradCompWeights ),2)/K;







