
%simple example file that runs distBoost on the protein dataset from the UCI repository.

if(~exist('simpleFlag','var'))

  disp('you can manually set simpleFlag, default value is now set to 1 (simple run)');
  disp('press any key to continue')
  pause
  simpleFlag=1;
end
disp('initializing...');
%add_boost_to_path


disp(['running on protein dataset'])
dataName= 'proteinData';

load(dataName);

[N d]= size(data);

% create constraints using a distributed learning scenario:

l= length(unique(labels));
qs= l*2;  %query size is about twice number of classes!
numberOfRealizations=1;
cComponentsPer= 0.9; %percentage of points in some equivalence constraints:

%generate constraints using the distributed learning scenario:
[ChunkletsMat, anti, AntiFull]= UCIconstraints(qs,cComponentsPer,labels,numberOfRealizations);

%determine number of labels:
M= length(classSizes);


%run DistBoost on data: (simple version)
if(simpleFlag)
  disp('running simple version demo: using the DistBoost function with minimal inputs');
%  [distMat, distFunc]= runDistBoost(data,labels,ChunkletsMat(1,:),anti{1});
  [outputMats ,Hyp]= DistBoost(data,ChunkletsMat(1,:),anti{1},labels); %labels is also optional!!!
  
  %show final distance matrix:
  imagesc(outputMats.final),colorbar;
  title(['DistBoost''s final distance matrix on the protein dataset']);

else  %full version
  % run DistBoost itself while controlling some of the Boosting parameters directly: (see DistBoost.m)
  disp('running full version demo: using the DistBoost function directly')           
  BSParams.WeakParams.M= M;
  BSParams.T=15;
  BSParams.printStatsFlag=1;
  BSParams.evaluateHypFlag=1;
  if(BSParams.evaluateHypFlag)
    BSParams.Knn=[1,5];
  end
  
  %call DistBoost:
  [distanceMats ,Hyp, NeighborPurity, KnnError, DevelopementStats]= DistBoost(data,ChunkletsMat(1,:),anti{1},labels,BSParams);
  
  %show final distance matrix:
  imagesc(distanceMats.final),colorbar;
  title(['DistBoost final distance matrix on the protein dataset']);
  
  %now show some evaluation statistics:
  s= sprintf('Cumulative Neighbor Purity results: \n');
  s= [s,sprintf('Euclidean        - ')];
  s= [s, sprintf('%.3f ',NeighborPurity(1,:))];
  s= [s,sprintf('\nWhitened         - ')];
  s= [s, sprintf('%.3f ',NeighborPurity(2,:))];
  s= [s,sprintf('\nRCA              - ')];
  s= [s, sprintf('%.3f ',NeighborPurity(3,:))];
  s= [s,sprintf('\nDistBoost ')];
  s= [s, sprintf('%.3f ',NeighborPurity(5,:))];
  

  s1= sprintf('\n\nK-nearest neighbor results for k= ');
  s1= [s1,sprintf('%d  ',BSParams.Knn)];
  s1= [s1,sprintf('\nEuclidean        - ')];
  s1= [s1, sprintf('%.3f  ',KnnError(1,:))];
  s1= [s1,sprintf('\nWhitened         - ')];
  s1= [s1, sprintf('%.3f  ',KnnError(2,:))];
  s1= [s1,sprintf('\nRCA              - ')];
  s1= [s1, sprintf('%.3f  ',KnnError(3,:))];
  s1= [s1,sprintf('\nDistBoost (t==T) - ')];
  s1= [s1, sprintf('%.3f  ',KnnError(5,:))];
  
  disp(s);
  disp(s1);
  
  %plot neighbor purity of the 5 compared methods:
  figure;
  hold on;
  colors='rgbck';
  for i=[1,2,3,5]
    plot(NeighborPurity(i,:),colors(i));
  end
  legend('Euclid','Whitened','RCA','DistBoost');
  title('Cumulative Neighbor Purity on Protein dataset using various distance functions');
  xlabel('Neighbors');
  ylabel('% correct neighbors');
end



